import io
import os
import unittest
from unittest.mock import Mock

from booknot.crawler.http_crawler import HttpCrawler
from booknot_tests.fixtures import clone_fixture


class TestHttpCrawler(unittest.TestCase):
    def setUp(self):
        self.session_mock = Mock()
        self._tested = HttpCrawler(self.session_mock)

    def test_fetch_should_parse_og_description(self):
        with clone_fixture("webpages") as directory:
            with io.open(os.path.join(directory, 'classic.html')) as filep:
                content = filep.read()

            # Assign
            request = Mock()
            request.content = content
            self.session_mock.get = Mock(return_value=request)

            # Acts
            result = self._tested.fetch("http://mock")

            # Assert
            self.assertEqual('Avant 10 JVMs, maintenant 300 microservices avec des runtimes transients. Le nombre de service à assembler pour construire nos applications augmente.Le SI se...', result.description)

    def test_fetch_should_parse_old_description(self):
        with clone_fixture("webpages") as directory:
            with io.open(os.path.join(directory, 'legacy.html')) as filep:
                content = filep.read()

            # Assign
            request = Mock()
            request.content = content
            self.session_mock.get = Mock(return_value=request)

            # Acts
            result = self._tested.fetch("http://mock")

            # Assert
            self.assertEqual('Avant 10 JVMs, maintenant 300 microservices avec des runtimes transients. Le nombre de service à assembler pour construire nos applications augmente.Le SI se...', result.description)

if __name__ == '__main__':
    unittest.main()



# pylint: disable=line-too-long
import click
import inquirer

from booknot.booknot_storage.booknot_storage import BooknotStorage

class InitApplication:

    def __init__(self, booknot_storage: BooknotStorage):
        self.booknot_storage = booknot_storage

    def run(self):

        if self.booknot_storage.exists():
            raise click.ClickException('.booknot already exists')

        if not self.booknot_storage.is_sphinx_present():
            questions = [
                inquirer.Confirm('init sphinx',
                                 message='This will bootstrap a workspace for sphinx in this directory, do you want to continue ?',
                                 default=False),
            ]

            answers = inquirer.prompt(questions)
            if answers['init sphinx']:
                questions = [
                    inquirer.Text('project',
                                     message='What the name of this booknot',
                                     default='Booknot'),
                    inquirer.Text('author',
                                     message='What the name of the author',
                                     default='me'),
                ]

                answers = inquirer.prompt(questions)
                self.booknot_storage.create_sphinx(answers['project'], answers['author'])

                click.echo(click.style("1. to render the booknot, use : make html", fg='yellow'))
                click.echo(click.style("2. to open the booknot after rendering, use a browser and open _build/html/index.html", fg='yellow'))
                self.booknot_storage.init_store()
                click.echo(click.style("3. to bookmark a page, use : booknot capture https://...", fg='yellow'))
        else:
            self.booknot_storage.init_store()
            click.echo(click.style("1. to bookmark a page, use : booknot capture https://...", fg='yellow'))



# pylint: disable=broad-except
import logging
import re
from datetime import datetime

import requests
from bs4 import BeautifulSoup

from booknot.crawler.crawler import Crawler
from booknot.crawler.metadata import Metadata


class HttpCrawler(Crawler):

    def __init__(self, session: requests.Session):
        self.session = session

    def fetch(self, url: str) -> Metadata:
        r = self.session.get(url)
        content = r.content
        return self.parse_content(url, content)

    def parse_content(self, url: str, content: str) -> Metadata:
        soup = BeautifulSoup(content, 'html.parser')
        title = soup.head.title.string
        description = self.extract_description(soup)

        now = datetime.now()
        return Metadata(url=url, title=title, description=description, capture_date=now)

    def extract_description(self, soup: BeautifulSoup):
        description = ''
        try:
            description_node = soup.head.find('meta', property="og:description")
            if description_node is None:
                description_node = soup.head.find(attrs={"name": re.compile(r'Description', re.I)})

            description = description_node["content"]
        except Exception as exception:
            logging.exception(exception)

        return description
